/****************************************************************************/
#include <sys/ioctl.h>
#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <linux/ioctl.h>
#include <errno.h>
#include <unistd.h>
#include <getopt.h>
#include <strings.h>
#include <time.h>

#include "ppi.h"
#include "jpeg_encoder.h"
#include "senddata.h"

#include <sys/mman.h>
#include <sys/stat.h>
#include <errno.h> 
#include <resolv.h>

/* Global variables declaration */
//int camera_out_format = QVGA_YUV422;
int camera_out_format = VGA_YUV422;


static int WIDTH;
static int HEIGHT;
static int IMG_SIZE;

static int fd_ppi;
static unsigned long video_in;
unsigned char *dma_buff;
unsigned char *in_buff;
unsigned char *out_buff;

static double msecond();

/* Setting for SAA7113  */
int set_saa7113_fmt()
{
                        
	WIDTH = 720;
	HEIGHT = 243;  //May be a bug. PAL 720x576, NTSC 720x488. Only one field to be encoded. 
	IMG_SIZE = WIDTH*HEIGHT*2;	
	return 0;
}

/* Setting for OV7640  */
int set_ov7640_fmt()
{

	 switch(camera_out_format){

                case QVGA_YUV422:
                        WIDTH  = 320;
                        HEIGHT = 240;
                        IMG_SIZE = WIDTH*HEIGHT*2;
                        break;
						
                case VGA_YUV422:
                        WIDTH  = 640;
                        HEIGHT = 480;
                        IMG_SIZE = WIDTH*HEIGHT*2;
                        break;

                default:
                        printf("Unkonw format!\n");
                        break;
        }
		
	/* Setting camera regs */
        switch(camera_out_format){

		case QVGA_YUV422:
		case VGA_YUV422:
			if(camera_out_format == VGA_YUV422)
				{
				ioctl(fd_ppi, CMD_PPI_SET_OV7640, 0x1404);		//1404:640*480
				ioctl(fd_ppi, CMD_PPI_SET_OV7640, 0x1581);
				}
			else if(camera_out_format == QVGA_YUV422)
                        	{
                        	ioctl(fd_ppi, CMD_PPI_SET_OV7640, 0x1424);		//1424:320*240
                        	ioctl(fd_ppi, CMD_PPI_SET_OV7640, 0x1580);
                        	}

		ioctl(fd_ppi, CMD_PPI_SET_OV7640, 0x1142);
		ioctl(fd_ppi, CMD_PPI_SET_OV7640, 0x1204);		
		
		ioctl(fd_ppi, CMD_PPI_SET_OV7640, 0x171A);
		ioctl(fd_ppi, CMD_PPI_SET_OV7640, (0x18<<8)+ 0x1A+640/4);	//0x18	//0x1a+640/4
		ioctl(fd_ppi, CMD_PPI_SET_OV7640, 0x1903);
		ioctl(fd_ppi, CMD_PPI_SET_OV7640, (0x1A<<8)+ 0x03+240);

		ioctl(fd_ppi, CMD_PPI_SET_OV7640, 0x2AA0);
		ioctl(fd_ppi, CMD_PPI_SET_OV7640, 0x2B7B);

		
                        break;

                default:
                        printf("Unkonw format!\n");
                        break;
        }

	return 0;
}


/****************************************************************************
*  Main loop
*
*****************************************************************************/
int main(int argc, char** argv)
{	
	int nOutputBytes;
	int yuv_width, yuv_height;
	double enctime;
//	double totaltime;	
	char devname[20]="/dev/ppi";

	static char *hostname;	
	struct sockaddr_in to;
	argc -=1;
	argv +=1;
	to.sin_family = AF_INET;
	//inet_ntoa(*(struct in_addr *)&to.sin_addr.s_addr));
	if (inet_aton(*argv, &to.sin_addr))
	{
		hostname = *argv;
        }
	else
	{
		printf("Host IP ERROR\n");
		printf("Usage: imgview pcIP\n");
		return -1;
	}

	/* open ppi driver device */
	printf("open %s \n",devname);
	if ((fd_ppi = open(devname, O_RDWR)) < 0)
	{
		printf("Device %s open error !\n",devname);
		exit(-1);
	}
	
        ioctl(fd_ppi, CMD_PPI_GET_VIDEOIN, &video_in);
	
	if(video_in == OV7640)
	{
		set_ov7640_fmt();
		printf("OV7640 is used!\n");
	}
	else if(video_in == SAA7113)
	{
		set_saa7113_fmt();
		printf("SAA7113 (PAL) is used!\n");
	}
	else
		printf("No valid video device is found!\n");
	
	
        /* Allocate meory for the raw image and BMP Header */
	out_buff = (unsigned char*)malloc(IMG_SIZE);

	/* Setting ppi parameters */	
	ioctl(fd_ppi, CMD_PPI_SET_PIXELS_PER_LINE, WIDTH*2);
	ioctl(fd_ppi, CMD_PPI_SET_LINES_PER_FRAME, HEIGHT);

	printf("WIDTH = %d , HEIGHT = %d \n",WIDTH,HEIGHT);
	
	/* Initialize socket */
	initsock(hostname,PORT_SOCKET_CLIENT,PORT_SOCKET_SERVER);

	/* Initialize JPEG encoder */        
	InitEncoder(80, YUV420_INPUT);

	while(1)
	{
		enctime = msecond();
		/* Read camera data */
		if( ( in_buff = mmap(NULL,IMG_SIZE,PROT_READ | PROT_WRITE , MAP_PRIVATE,fd_ppi,0)) == MAP_FAILED)		//MAP_FAILED=-1
		{			
			printf(" mmap error ! errno=%d \n",errno);
			exit(0);
		}
		enctime = msecond() - enctime;
		printf("get data=%6.2fms\n",enctime);	
		enctime = msecond();
		/* ITU656 --> YUV420 */

		if(video_in == OV7640)
			yuv4222yuv420_cmos(WIDTH, HEIGHT, in_buff, out_buff);			
		else	
		{
			//yuv4222yuv420(WIDTH, HEIGHT, in_buff, out_buff);	//720*288 => 352*288
			//WIDTH = 352;			
			yuv4222yuv420_cmos(WIDTH, HEIGHT, in_buff, out_buff);		//720*288 => 720*288
		}

		
		yuv_width = WIDTH;
		yuv_height = HEIGHT;
		
		enctime = msecond() - enctime;
		printf("yuv convert = %6.2fms\n",enctime);
		enctime = msecond();

		CompressImage(out_buff,in_buff,yuv_width,yuv_height,&nOutputBytes);
		enctime = msecond() - enctime;
		printf("jpeg = %6.2fms\n",enctime);		
		enctime = msecond();
		
		senddata(in_buff,nOutputBytes);
		printf("send bytes %d\n", nOutputBytes);
		enctime = msecond() - enctime;
		printf("send = %6.2fms\n",enctime);

		//totaltime = msecond() - totaltime;
		//printf("total = %6.2fms\n",totaltime);
		//totaltime = msecond();
	
		munmap(in_buff,IMG_SIZE);
	}

	/* Free memory */	
	if(out_buff)free(out_buff);
	//if(in_buff)free(in_buff);
	
	/* Free system sources */
	if(fd_ppi)close(fd_ppi);
	
	printf("Bye bye!\n");
	return 0;
}

static double msecond()
{
        struct timeval tv;
        gettimeofday(&tv, 0);
        return (tv.tv_sec * 1.0e3 + tv.tv_usec * 1.0e-3);
}


